/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.ant;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.net.URL;
import java.sql.SQLException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import biz.hammurapi.sql.SQLProcessor;

/**
 * Executes SQL script
 * 
 * @ant.task name="Script" category="Common"
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.1 $
 */
public class Script extends Task {
	private String script="";
	private String delimiter=";";
	private ConnectionEntry connectionEntry;
	
	/**
	 * Connection
	 * @ant.not-required
	 * @param connectionEntry
	 */
	public void addConfiguredConnection(ConnectionEntry connectionEntry) {
		this.connectionEntry=connectionEntry;
	}
	
	/**
	 * Statements delimiter (single character). Defaults to ';'
	 * @ant.not-required
	 * @param delimiter
	 */
	public void setDelimiter(String delimiter) {
		this.delimiter=delimiter;
	}
	
	/**
	 * Script text. Required if file is not set.
	 * @ant.not-required
	 * @param text
	 */
	public void  addText(String text) {
		script+=text;
	}
	
	private File file;
	private URL url;
	private String resource;
		
	/**
	 * Script file.  One and only one of 'file', 'url', or 'resource' is required.
	 * @ant.required
	 * @param file
	 */
	public void setFile(File file) {
		if (url!=null) {
			throw new BuildException("File and URL are mutually exclusive. URL already set");
		}
		if (resource!=null) {
			throw new BuildException("File and resource are mutually exclusive. Resource already set");
		}
		this.file = file;
	}
	
	/**
	 * Script classloader resource.  One and only one of 'file', 'url', or 'resource' is required.
	 * @ant.required
	 * @param file
	 */
	public void setResource(String resource) {
		if (url!=null) {
			throw new BuildException("Resource and URL are mutually exclusive. URL already set");
		}
		if (file!=null) {
			throw new BuildException("File and resource are mutually exclusive. File already set");
		}
		this.resource = resource;
	}
	
	/**
	 * Script URL. One and only one of 'file', 'url', or 'resource' is required. 
	 * @ant.required
	 * @param url
	 */
	public void setUrl(URL url) {
		if (file!=null) {
			throw new BuildException("File and URL are mutually exclusive. File already set");
		}		
		if (resource!=null) {
			throw new BuildException("Resource and URL are mutually exclusive. Resource already set");
		}		
		this.url = url;
	}
	
	public void execute(SQLProcessor processor) throws IOException, SQLException {
		if (script.trim().length()>0) {
			processor.executeScript(new StringReader(script), delimiter.charAt(0));
		}
		
		if (file!=null) {
			processor.executeScript(new FileReader(file), delimiter.charAt(0));
		}
		
		if (url!=null) {
			processor.executeScript(new InputStreamReader(url.openStream()), delimiter.charAt(0));
		}
		
		if (resource!=null) {
			InputStream resourceAsStream = getClass().getClassLoader().getResourceAsStream(resource);
			if (resourceAsStream==null) {
				throw new BuildException("Resource not found: "+resource);
			}
			processor.executeScript(new InputStreamReader(resourceAsStream), delimiter.charAt(0));
		}		
	}
	
	public void execute() throws BuildException {
		try {
			execute(new SQLProcessor(connectionEntry.getDataSource(),null));
		} catch (IOException e) {
			throw new  BuildException(e);
		} catch (SQLException e) {
			throw new  BuildException(e);
		}
	}
}
