/*
 * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.convert;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.lang.reflect.Constructor;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import biz.hammurapi.config.Context;


/**
 * @author Pavel Vlasov
 * @version $Revision: 1.7 $
 */
public class FoundationAtomicConvertersBundle implements AtomicConvertersBundle {

	private List converters=new ArrayList();
	
	public Collection getConverters() {
		return Collections.unmodifiableCollection(converters);
	}

    /**
     * Creates new composite converter populated by default with some generic converters.
     */
    public FoundationAtomicConvertersBundle() {
		addConverter(Number.class, byte.class, "byteValue", null);
		addConverter(Number.class, double.class, "doubleValue", null);
		addConverter(Number.class, float.class, "floatValue", null);
		addConverter(Number.class, int.class, "intValue", null);
		addConverter(Number.class, long.class, "longValue", null);
		addConverter(Number.class, short.class, "shortValue", null);
		
		addConverter(Number.class, Byte.class, "byteValue", null);
		addConverter(Number.class, Double.class, "doubleValue", null);
		addConverter(Number.class, Float.class, "floatValue", null);
		addConverter(Number.class, Integer.class, "intValue", null);
		addConverter(Number.class, Long.class, "longValue", null);
		addConverter(Number.class, Short.class, "shortValue", null);
		
		converters.addAll(ReflectionConverter.discoverConstructorConverters(String.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(Integer.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(Long.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(Double.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(Float.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(Byte.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(Short.class));
		converters.addAll(ReflectionConverter.discoverConstructorConverters(BigDecimal.class));
		
		try {
			addConverter(File.class, Reader.class, null, FileReader.class.getConstructor(new Class[] {File.class}));
			addConverter(File.class, InputStream.class, null, FileInputStream.class.getConstructor(new Class[] {File.class}));
			addConverter(InputStream.class, Reader.class, null, InputStreamReader.class.getConstructor(new Class[] {InputStream.class}));
		} catch (SecurityException e) {
			throw new ConversionException(e);
		} catch (NoSuchMethodException e) {
			throw new ConversionException(e);
		}
		
		converters.add(new AtomicConverterBase(String.class, boolean.class) {

			public Object convert(Object source, Converter master, Context context) {
				if (source==null) {
					return Boolean.FALSE;
				}
				
				String str = ((String) source).trim();
				if (str.length()==0 || "no".equalsIgnoreCase(str) || "false".equalsIgnoreCase(str) || "0".equals(source) || "".equals(source)) {
					return Boolean.FALSE;
				} else if ("yes".equalsIgnoreCase(str) || "true".equalsIgnoreCase(str) || "1".equals(source) || "on".equals(source)) {
					return Boolean.TRUE;
				} 
				
				return null;
			}
			
		});
		
		converters.add(new AtomicConverterBase(String.class, byte.class) {

			public Object convert(Object source, Converter master, Context context) {
				return new Byte((String) source);
			}
			
		});
		
		converters.add(new AtomicConverterBase(String.class, double.class) {

			public Object convert(Object source, Converter master, Context context) {
				return new Double((String) source);
			}
			
		});
		
		converters.add(new AtomicConverterBase(String.class, float.class) {

			public Object convert(Object source, Converter master, Context context) {
				return new Float((String) source);
			}
			
		});
		

		converters.add(new AtomicConverterBase(String.class, int.class) {

			public Object convert(Object source, Converter master, Context context) {
				return new Integer((String) source);
			}
			
		});
		

		converters.add(new AtomicConverterBase(String.class, long.class) {

			public Object convert(Object source, Converter master, Context context) {
				return new Long((String) source);
			}
			
		});
		

		converters.add(new AtomicConverterBase(String.class, short.class) {

			public Object convert(Object source, Converter master, Context context) {
				return new Short((String) source);
			}
			
		});
				
		converters.add(new AtomicConverterBase(String.class, Class.class) {

			public Object convert(Object source, Converter master, Context context) {
				try {
					return Class.forName((String) source);
				} catch (ClassNotFoundException e) {
					throw new ConversionException(e);
				}
			}
			
		});		
		
    }
		
	private void addConverter(Class source, Class target, String accessor, Constructor constructor) {
		try {
			converters.add(new ReflectionConverter(accessor==null ? null : source.getMethod(accessor, null), constructor));
		} catch (Exception e) {
			throw new ConversionException(e);
		}
	}
		
}
