/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.properties;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

/**
 * Wrapper around a property set to get access to a sub-set of properties based on prefix.
 * @author Pavel
 */
public class PropertySubSet implements PropertySet {
	
	private PropertySet master;
	private String prefix;

	public PropertySubSet(PropertySet master, String prefix) {
		this.master = master;
		this.prefix = prefix;
	}

	public Set getPropertyNames() {
		Set ret = new HashSet();
		
		Iterator nit = master.getPropertyNames().iterator();
		while (nit.hasNext()) {
			String name = (String) nit.next(); 
			if (name.startsWith(prefix)) {
				ret.add(name.substring(prefix.length()));
			}
		}
		return ret;
	}

	public PropertySet getSubset(String prefix) {		
		return new PropertySubSet(this, prefix);
	}

	public boolean remove(String name) {
		return master.remove(prefix+name);
	}

	public void set(String name, Object value) {
		master.set(prefix+name, value);
	}

	public Object get(String name) {
		Object ret = master.get(prefix+name);
		if (ret==null) {
			Iterator esit = mounts.entrySet().iterator();
			while (esit.hasNext()) {
				Entry entry = (Entry) esit.next();				
				if (name.startsWith((String) entry.getKey())) {
					ret = ((PropertySet) entry.getValue()).get(name.substring(((String) entry.getKey()).length()));
					if (ret!=null) {
						return ret;
					}
				}
			}
		}
		return ret;
	}

	public void clear() {
		Iterator nit = master.getPropertyNames().iterator();
		while (nit.hasNext()) {
			String name = (String) nit.next();
			if (name.startsWith(prefix)) {
				master.remove(name);
			}
		}		
	}

	private Map mounts = new HashMap();

	public void mount(String prefix, PropertySet source) {
		mounts.put(prefix, source);		
	}

	public void setAll(PropertySet source) {
		Iterator pnit = source.getPropertyNames().iterator();
		while (pnit.hasNext()) {
			String pName = (String) pnit.next();
			set(pName, source.get(pName));
		}		
	}		
	
	/**
	 * @return true if other property set entries are equal to this property set entries. 
	 * Other Property set attribute
	 */
	public boolean compareProperties(PropertySet otherSet) {
		Set myNames = new HashSet(getPropertyNames());
		Set otherNames = new HashSet(otherSet.getPropertyNames());
		if (!myNames.equals(otherNames)) {
			return false;
		}
		
		Iterator mnit = myNames.iterator();
		while (mnit.hasNext()) {
			String pName =  (String) mnit.next();
			Object myValue = get(pName);
			Object otherValue = otherSet.get(pName);
			if (myValue==null) {
				if (otherValue!=null) {
					return false;
				}
			} else if (!myValue.equals(otherValue)) {
				return false;
			}			
		}
		
		return true;
	}

	public boolean containsAll(PropertySet subSet) {
		Set myNames = new HashSet(getPropertyNames());
		Set subsetNames = new HashSet(subSet.getPropertyNames());
		if (!myNames.containsAll(subsetNames)) {
			return false;
		}
		
		Iterator snit = subsetNames.iterator();
		while (snit.hasNext()) {
			String pName = (String) snit.next();
			Object myValue = get(pName);
			Object subsetValue = subSet.get(pName);
			if (myValue==null) {
				if (subsetValue!=null) {
					return false;
				}
			} else if (!myValue.equals(subsetValue)) {
				return false;
			}			
		}
		
		return true;
	}

	public Object get(String name, Object defaultValue) {
		Object ret = get(name);
		return ret == null ? defaultValue : ret;
	}	
}
