package biz.hammurapi.sql.hsqldb;

import org.hsqldb.Server;
import org.hsqldb.ServerConfiguration;
import org.hsqldb.ServerConstants;
import org.hsqldb.lib.FileUtil;
import org.hsqldb.persist.HsqlProperties;

/**
 * Helper class to start HSQLDB server as Windows service.
 * @author Pavel
 *
 */
public class HsqldbService {
	
	private static Server server;

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		if (args.length==0 || "start".equals(args[0])) {
			if (server==null) {
		        String propsPath = FileUtil.getDefaultInstance().canonicalOrAbsolutePath("server");
		        HsqlProperties fileProps = ServerConfiguration.getPropertiesFromFile(propsPath);
		        HsqlProperties props = fileProps == null ? new HsqlProperties() : fileProps;
		        HsqlProperties stringProps = HsqlProperties.argArrayToProps(args, ServerConstants.SC_KEY_PREFIX);

		        if (stringProps != null) {
		            if (stringProps.getErrorKeys().length != 0) {
		            	System.exit(2);
		            }

		            props.addProperties(stringProps);
		        }

		        ServerConfiguration.translateDefaultDatabaseProperty(props);
		        ServerConfiguration.translateDefaultNoSystemExitProperty(props);

		        server = new Server();
	            server.setProperties(props);
		        server.start();				
			} else {
				System.err.println("Server is already running.");
				System.exit(1);
			}
		}

		if (args.length!=0 && "stop".equals(args[0])) {
			if (server==null) {
				System.err.println("Server is not running.");
				System.exit(1);				
			} else {
				server.shutdown();
			}			
		}
	}

}
