package biz.hammurapi.sql.syntax;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import biz.hammurapi.sql.Variant;

/**
 * Equalt
 * @author Pavel
 *
 */
public class Between implements StatementFragment, Serializable {
	
	private Variant lower;
	private Variant upper;
	private String columnName;
	
	public Between(String columnName, boolean lower, boolean upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, byte lower, byte upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, char lower, char upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, double lower, double upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, float lower, float upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, int lower, int upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, long lower, long upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, String lower, String upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, Object lower, Object upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public Between(String columnName, Object lower, Object upper, int sqlType) {
		this.lower = new Variant(lower, sqlType);
		this.upper = new Variant(upper, sqlType);
		this.columnName = columnName;
	}

	public Between(String columnName, short lower, short upper) {
		this.lower = new Variant(lower);
		this.upper = new Variant(upper);
		this.columnName = columnName;
	}

	public String toSqlString() {		
		return columnName + " BETWEEN ? AND ?";
	}

	public int parameterize(PreparedStatement ps, int idx) throws SQLException {
		return upper.parameterize(ps, lower.parameterize(ps, idx));
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((columnName == null) ? 0 : columnName.hashCode());
		result = prime * result + ((lower == null) ? 0 : lower.hashCode());
		result = prime * result + ((upper == null) ? 0 : upper.hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Between other = (Between) obj;
		if (columnName == null) {
			if (other.columnName != null)
				return false;
		} else if (!columnName.equals(other.columnName))
			return false;
		if (lower == null) {
			if (other.lower != null)
				return false;
		} else if (!lower.equals(other.lower))
			return false;
		if (upper == null) {
			if (other.upper != null)
				return false;
		} else if (!upper.equals(other.upper))
			return false;
		return true;
	}

}
