/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.swing;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.TreeNode;

import biz.hammurapi.convert.Converter;
import biz.hammurapi.convert.ConvertingService;

/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.1 $
 */
public class BeanVisualizer {
	
	private static final Logger logger = Logger.getLogger(BeanVisualizer.class.getName());
	
	private static boolean isPrimitive(Class klass) {
		return String.class.equals(klass) 
		|| Number.class.isAssignableFrom(klass) 
		|| Character.class.equals(klass)
		|| Boolean.class.equals(klass)
		|| double.class.equals(klass) 
		|| float.class.equals(klass) 
		|| long.class.equals(klass) 
		|| int.class.equals(klass) 
		|| short.class.equals(klass) 
		|| byte.class.equals(klass) 
		|| char.class.equals(klass) 
		|| boolean.class.equals(klass); 
 	}
	
	public Visualizable convert(final Object obj, Converter master) {
		return new Visualizable() {

			public TreeNode toTreeNode(TreeNode parent, String title) {
				return new LazyTreeNodeTableVisualizable(parent, title+" ["+obj.getClass().getName()+"] "+obj) {

					protected List loadChildren() {
						List ret = new ArrayList();
					    Class beanClass=obj.getClass();
					    Method[] methods = beanClass.getMethods();
					    for (int i=0; i<methods.length; i++) {
					    	// getXXX() methods. Object.getClass() is not included.
					    	if (!(methods[i].getDeclaringClass().equals(Object.class)) 
					    			&& methods[i].getName().startsWith("get") 
					    			&& methods[i].getParameterTypes().length==0 
					    			&& !Modifier.isStatic(methods[i].getModifiers())
					    			&& methods[i].isSynthetic()
					    			&& !isPrimitive(methods[i].getReturnType())) {
					    		try {
									Object value=methods[i].invoke(obj, null);
									Visualizable vs = (Visualizable) ConvertingService.convert(value, Visualizable.class);
									ret.add(vs.toTreeNode(this, methods[i].getName().substring(3)+" ("+methods[i].getReturnType().getName()+")"));
								} catch (Exception e) {
									logger.log(Level.WARNING, "Could not retrieve value from "+methods[i], e);								
								}
					    	}					    						    	
					    }
					    
					    Field[] fields = beanClass.getFields();
					    for (int i=0; i<fields.length; i++) {
				    		try {
				    			if (!isPrimitive(fields[i].getType()) 
				    					&& !Modifier.isStatic(methods[i].getModifiers())
				    					&& !fields[i].isSynthetic()) {
									Object value=fields[i].get(obj);
									Visualizable vs = (Visualizable) ConvertingService.convert(value, Visualizable.class);
									ret.add(vs.toTreeNode(this, fields[i].getName()+" ("+fields[i].getType().getName()+")"));
				    			}
							} catch (Exception e) {
								logger.log(Level.WARNING, "Could not retrieve value from "+methods[i], e);								
							}
					    }
						
						return ret;
					}

					public TableModel toTable() {
						int rowCount=2;
						DefaultTableModel tm=new DefaultTableModel(rowCount,4);			
						tm.setColumnIdentifiers(new String[] {"Property", "Type", "Value"});
						
					    Class beanClass=obj.getClass();
					    
						tm.setValueAt("this", 0, 0);
						tm.setValueAt(beanClass.getName(), 0, 1);
						tm.setValueAt(obj, 0, 2);
						
						tm.setValueAt("Hash code", 1, 0);
						tm.setValueAt("int", 1, 1);
						tm.setValueAt(Integer.toString(obj.hashCode(), Character.MAX_RADIX), 1, 2);
						
					    Method[] methods = beanClass.getMethods();
					    for (int i=0; i<methods.length; i++) {
					    	// getXXX() methods. Object.getClass() is not included.
					    	if (!(methods[i].getDeclaringClass().equals(Object.class)) 
					    			&& methods[i].getName().startsWith("get") 
					    			&& methods[i].getParameterTypes().length==0 
					    			&& !Modifier.isStatic(methods[i].getModifiers())
					    			&& methods[i].isSynthetic()
					    			&& isPrimitive(methods[i].getReturnType())) {
					    		try {
									Object value=methods[i].invoke(obj, null);
									tm.setRowCount(++rowCount);
									int idx = rowCount-1;
									tm.setValueAt(methods[i].getName().substring(3), idx, 0);
									if (value==null) {
										tm.setValueAt("(null)", idx, 1);
										tm.setValueAt("(null)", idx, 2);
									} else {
										tm.setValueAt(value.getClass().getName(), idx, 1);
										tm.setValueAt(value, idx, 2);
									}
								} catch (Exception e) {
									logger.log(Level.WARNING, "Could not retrieve value from "+methods[i], e);								
								}
					    	}					    						    	
					    }
					    
					    Field[] fields = beanClass.getFields();
					    for (int i=0; i<fields.length; i++) {
				    		try {
				    			if (isPrimitive(fields[i].getType()) 
				    					&& !Modifier.isStatic(methods[i].getModifiers()) 
				    					&& !fields[i].isSynthetic()) {
									Object value=fields[i].get(obj);								
									tm.setRowCount(++rowCount);
									int idx = rowCount-1;
									tm.setValueAt(fields[i].getName(), idx, 0);
									if (value==null) {
										tm.setValueAt("(null)", idx, 1);
										tm.setValueAt("(null)", idx, 2);
									} else {
										tm.setValueAt(value.getClass().getName(), idx, 1);
										tm.setValueAt(value, idx, 2);
									}
				    			}
							} catch (Exception e) {
								logger.log(Level.WARNING, "Could not retrieve value from "+methods[i], e);								
							}
					    }
					    
						return tm;
					}
					
				};
			}
			
		};
	}
	
}
