/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.swing;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.TreeNode;

import biz.hammurapi.convert.Converter;

/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.1 $
 */
public class MapVisualizer {

	public Visualizable convert(final Map map, final Converter master) {
		return new Visualizable() {

			public TreeNode toTreeNode(TreeNode parent, String title) {
				return new LazyTreeNodeTableVisualizable(parent, title) {

					protected List loadChildren() {
						ArrayList ret = new ArrayList();
						Iterator it = map.entrySet().iterator();				
						for (int i=0; it.hasNext(); ++i) {
							Visualizable vs = (Visualizable) master.convert(it.next(), Visualizable.class, null);
							ret.add(vs.toTreeNode(this, "entry "+i));
						}
						return ret;
					}

					public TableModel toTable() {
						DefaultTableModel tm=new DefaultTableModel(2,2);
						tm.setColumnIdentifiers(new String[] {"Property", "Value"});
						tm.setValueAt("type", 0, 0);
						tm.setValueAt(map.getClass().getName(), 0, 1);
						tm.setValueAt("size", 1, 0);
						tm.setValueAt(String.valueOf(map.size()), 1, 1);
						
						return tm;
					}
					
				};
			}
			
		};
	}

	public Visualizable convert(final Map.Entry entry, final Converter master) {
		return new Visualizable() {

			public TreeNode toTreeNode(TreeNode parent, String title) {
				return new LazyTreeNodeTableVisualizable(parent, title) {
		
					protected List loadChildren() {
						List ret = new ArrayList();
						Visualizable kvs = (Visualizable) master.convert(entry.getKey(), Visualizable.class, null);
						ret.add(kvs.toTreeNode(this, "key"));
						Visualizable vvs = (Visualizable) master.convert(entry.getKey(), Visualizable.class, null);
						ret.add(vvs.toTreeNode(this, "value"));
						return ret;
					}
		
					public TableModel toTable() {
						DefaultTableModel tm=new DefaultTableModel(2,3);
						tm.setColumnIdentifiers(new String[] {"Member", "Type", "Value"});
						tm.setValueAt("key", 0, 0);
						tm.setValueAt(entry.getKey()==null ? "(null)" : entry.getKey().getClass().getName(), 0, 1);
						tm.setValueAt(entry.getKey()==null ? "(null)" : entry.getKey(), 0, 2);
						
						tm.setValueAt("value", 1, 0);
						tm.setValueAt(entry.getValue()==null ? "(null)" : entry.getValue().getClass().getName(), 1, 1);
						tm.setValueAt(entry.getValue()==null ? "(null)" : entry.getValue(), 1, 2);
						
						return tm;
					}
					
					
				};
			}
		};
	}
}
